
#include <stdlib.h>
#include "string.h"
#include "math.h"
#include "iostream.h"
#include "PDB.h"
#include "fstream.h"
#include "Alignment.h"


Alignment::~Alignment()
{
	if (bAligned)
	{
		for (short i = 0; i < m_Length1 + 1; i++)
		{
			delete [] m_ScoreMatrix[i];
		}
		delete [] m_ScoreMatrix;
		delete [] m_LocalMaxArr;
	
		if (m_CurrentAlignment != -1)
		{
			delete [] m_shAligned1;
			delete [] m_shAligned2;

			delete [] m_shMap1;
			delete [] m_shMap2;
		}
	}
}

void Alignment::Align(short MaxAligns)
{
	if (bAligned)
		return;

	const short NumOfAligns = MaxAligns;

	short i, j, m, n;
	m_ScoreMatrix = (struct Trace**)new struct Trace*[m_Length1+1];
	for (i = 0; i <= m_Length1; i++)
	{
		m_ScoreMatrix[i] = new struct Trace[m_Length2+1];
	}

	m_LocalMaxArr = new struct LocalMax[NumOfAligns];

	for (n = 0; n < NumOfAligns; n++)
	{
		m_LocalMaxArr[n].Score = 0;
		m_LocalMaxArr[n].i = 0;
		m_LocalMaxArr[n].j = 0;
	}


	// Initialization Step
	m_Matches = 0;
	m_Mismatches = 0;
	m_Gaps = 0;

	for (i = 0; i < m_Length1+1; i++)
	{
		m_ScoreMatrix[i][0].gaptype = GAP_NONE;
		m_ScoreMatrix[i][0].value = 0;
	}

	for (j = 0; j < m_Length2+1; j++)
	{
		m_ScoreMatrix[0][j].gaptype = GAP_NONE;
		m_ScoreMatrix[0][j].value = 0;
	}
	// Fill In Scoring Matrix
	// and find MaxScore
	for (i = 1; i <= m_Length1; i++)
	{
		for (j = 1; j <= m_Length2; j++)
		{
			struct Trace tr;
		
			short diag = m_ScoreMatrix[i-1][j-1].value + Match(m_MaxDiff, m_MatchScore, m_MismatchPenalty, i-1, j-1);
			short hGap = m_ScoreMatrix[i][j-1].value + m_GapPenalty;
			short vGap = m_ScoreMatrix[i-1][j].value + m_GapPenalty;
			if ((diag >= hGap) && (diag >= vGap))
			{
				tr.gaptype = GAP_DIAG;
				if (diag > 0)
					tr.value = diag;
				else
					tr.value = 0;
			}
			else if ((hGap >= diag) && (hGap >= vGap))
			{
				tr.gaptype = GAP_H;
				if (hGap > 0)
					tr.value = hGap;
				else
					tr.value = 0;
			}
			else
			{
				tr.gaptype = GAP_V;
				if (vGap > 0)
					tr.value = vGap;
				else
					tr.value = 0;
			}
			m_ScoreMatrix[i][j].gaptype = tr.gaptype;
			m_ScoreMatrix[i][j].value = tr.value;
			if (tr.value > 0)
			{
				short check = 0;
				for (n = 0; n < NumOfAligns; n++)
				{
					check = check || (tr.value > m_LocalMaxArr[n].Score);
					if (check)
						break;
				}
				if (check)
				{
					for (m = NumOfAligns-1; m > n; m--)
					{
						m_LocalMaxArr[m] = m_LocalMaxArr[m-1];
					}
					m_LocalMaxArr[m].Score = tr.value;
					m_LocalMaxArr[m].i = i;
					m_LocalMaxArr[m].j = j;
				}
			}
		}
	}
	bAligned = 1;
	m_CurrentAlignment = -1;
	
}

float Alignment::GetAlignment(short num, short* indArr1, short* indArr2)
{
	if (!bAligned)
		return 0;
	// Do Trace-Back step 
	short seq_ind;
	short counter = 0;
	m_Matches = 0;
	m_Mismatches = 0;
	m_Gaps = 0;
	seq_ind = m_Length1 + m_Length2;
	if (seq_ind <= 0)
		return 0;
	if (m_CurrentAlignment != -1)
	{
		delete [] m_shAligned1;
		delete [] m_shAligned2;
		delete [] m_shMap1;
		delete [] m_shMap2;
		delete [] m_chGaps;
	}

	m_shAligned1 = new short[seq_ind];
	m_shAligned2 = new short[seq_ind];
	m_shMap1 = new short[seq_ind];
	m_shMap2 = new short[seq_ind];
	m_chGaps = new char[seq_ind];

	short istart = m_LocalMaxArr[num].i;
	short jstart = m_LocalMaxArr[num].j;
	short i, j;
	for (i = istart, j = jstart; m_ScoreMatrix[i][j].value > 0; )
	{
		switch(m_ScoreMatrix[i][j].gaptype)
		{
		case GAP_DIAG:
			m_shAligned1[counter] = m_seq1[i-1];
			m_shAligned2[counter] = m_seq2[j-1];
			m_shMap1[counter] = indArr1[i-1];
			m_shMap2[counter] = indArr2[j-1];
			if (Match(m_MaxDiff, m_MatchScore, m_MismatchPenalty, i-1, j-1) > 0)
			{
				m_chGaps[counter] = '-';
				m_Matches++;
			}
			else
			{
				m_chGaps[counter] = ' ';
				m_Mismatches++;
			}

			i--;
			j--;
			break;

		case GAP_H:
			m_shAligned1[counter] = -1;
			m_shAligned2[counter] = m_seq2[j-1];
			m_shMap1[counter] = -1;
			m_shMap2[counter] = indArr2[j-1];
			m_chGaps[counter] = ' ';
			m_Gaps++;
			j--;
			break;

		case GAP_V:
			m_shAligned1[counter] = m_seq1[i-1];
			m_shAligned2[counter] = -1;
			m_shMap1[counter] = indArr1[i-1];
			m_shMap2[counter] = -1;
			m_chGaps[counter] = ' ';
			m_Gaps++;
			i--;
			break;

		default:
			break;
		}
		counter++;
	}

	m_MatchLength = counter - 1;
	m_CurrentAlignment = num;

	
	float MismatchCoeff = ((float)m_MismatchPenalty/(float)(abs(m_GapPenalty)+m_MismatchPenalty));
	float GapCoeff = ((float)abs(m_GapPenalty)/(float)(abs(m_GapPenalty)+m_MismatchPenalty));
	return (((float)m_Matches - MismatchCoeff*(float)m_Mismatches - GapCoeff*(float)m_Gaps)/(float)counter);
}

short Alignment::ExportPDB(char* pdb_in1, char* pdb_in2, char* pdb_out1, char* pdb_out2)
{
	ifstream ifs1, ifs2;
	ofstream ofs1, ofs2;
	ifs1.open(pdb_in1, ios::in);
	ifs2.open(pdb_in2, ios::in);
	ofs1.open(pdb_out1, ios::out);
	ofs2.open(pdb_out2, ios::out);
	
	if ((!ifs1.good()) || (!ifs2.good()) || (!ofs1.good()) || (!ofs2.good()))
	{
		return -1;
	}
	char rec[81];
	ifs1.getline(rec, 81, '\n');
	short i = m_MatchLength;
	while (m_chGaps[i] != '-')
		i--;
	short lastIndex = -1;
	while (!(ifs1.eof()))
	{
		if (PDB::isATOMrec(rec) && ((PDB::atomChainId(rec) == 'A') || (PDB::atomChainId(rec) == ' ')) &&
			(lastIndex != -1) && (PDB::atomResidueIndex(rec) != lastIndex))
		{
			i--;
			while (m_chGaps[i] != '-')
				i--;
			lastIndex = -1;
		}
		// Query the line
		// chain A only is scanned
		if (PDB::isATOMrec(rec) && (PDB::atomResidueIndex(rec) == m_shMap1[i]) &&
		   ((PDB::atomChainId(rec) == 'A') || (PDB::atomChainId(rec) == ' ')))
		{
			lastIndex = PDB::atomResidueIndex(rec);
			
			rec[21] = 'A';
			ofs1 << rec << endl;
		}
		ifs1.getline(rec, 81, '\n');
	}
	ifs1.close();
	ofs1.close();

	ifs2.getline(rec, 81, '\n');
	i = m_MatchLength;
	while (m_chGaps[i] != '-')
		i--;
	lastIndex = -1;
	while (!(ifs2.eof()))
	{
		if (PDB::isATOMrec(rec) && ((PDB::atomChainId(rec) == 'A') || (PDB::atomChainId(rec) == ' ')) &&
			(lastIndex != -1) && (PDB::atomResidueIndex(rec) != lastIndex))
		{
			i--;
			while (m_chGaps[i] != '-')
				i--;
			lastIndex = -1;
		}
		// Query the line
		// chain A only is scanned
		if (PDB::isATOMrec(rec) && (PDB::atomResidueIndex(rec) == m_shMap2[i]) &&
		   ((PDB::atomChainId(rec) == 'A') || (PDB::atomChainId(rec) == ' ')))
		{
			lastIndex = PDB::atomResidueIndex(rec);

			rec[21] = 'B';
			ofs2 << rec << endl;
		}
		ifs2.getline(rec, 81, '\n');
	}
	ifs2.close();
	ofs2.close();

	return 0;
}


short Alignment::Match(short MaxDiff, short MatchScore, short MismatchPenalty, short i, short j)
{
	if (abs(m_seq1[i]-m_seq2[j]) <= MaxDiff)
		return MatchScore;
	else 
		return (-MismatchPenalty);
}

ostream& operator<<(ostream& os, Alignment& al)
{
	if (!al.bAligned)
	{
		os << "Empty" << endl;
		return os;
	}

	os << "Match Length: " << al.m_MatchLength << endl;
	os << "I" << "\t " << "\tII" << endl;
	for (short i = al.m_MatchLength; i >= 0; i--)
	{
		if (al.m_shAligned1[i] == -1)
			os << " ";
		else
			os << al.m_shAligned1[i];

		os << "\t" << al.m_chGaps[i] << "\t";
		
		if (al.m_shAligned2[i] == -1)
			os << " ";
		else
			os << al.m_shAligned2[i];

		os << endl;
	}

	return os;
}
