
#ifndef _ALIGNMENT_H
#define _ALIGNMENT_H

#include "iostream.h"

class Alignment
{
// Public methods
public:

	// Construction
	Alignment(short* seq1, short* seq2, short Length1, short Length2, short MaxDiff, short MatchScore, short MismatchPenalty, short GapPenalty) : 
			  m_seq1(seq1),
			  m_seq2(seq2),
			  m_Length1(Length1),
			  m_Length2(Length2),
			  m_MaxDiff(MaxDiff),
			  m_MatchScore(MatchScore),
			  m_MismatchPenalty(MismatchPenalty),
			  m_GapPenalty(GapPenalty),
			  bAligned(0)
			  {
			  };

	// Destruction
	~Alignment();

	// Calculate Sequence alignment (NW algorithm) and return Quality ratio
	void Align(short MaxAligns);

	// Get the specified (by id) alignment - eturns quality of match
	float GetAlignment(short num, short* indArr1, short* indArr2);

	// Build PDB file with 2 chains that will represent the alignment
	// return 0 if OK
	short ExportPDB(char* pdb_in1, char* pdb_in2, char* pdb_out1, char* pdb_out2);
	
	// Output the Aligned Sequences
	friend ostream& operator<<(ostream& os, Alignment& al);

// Private Methods
private:
	// Calculate the Match Score / Mismatch Penalty
	short Match(short MaxDiff, short MatchScore, short MismatchPenalty, short i, short j);
	// Calculate the best ancestor for trace-back step
	//struct Trace* MaxPrev(short diag, short hGap, short vGap, short i, short j);

// Private Structure declaration
private:
	enum GAP
	{
		GAP_NONE,
		GAP_DIAG,
		GAP_H,
		GAP_V
	};

	struct Trace
	{
		GAP gaptype;
		short value;
	};

	struct LocalMax
	{
		short Score;
		short i;
		short j;
	};

// Private Member Variables
private:
	short* m_seq1; // First sequence
	short* m_seq2; // Second sequence
	char* m_chGaps;
	const short m_Length1; // Length of the first sequence
	const short m_Length2; // Length of the second sequence

	short* m_shAligned1; // Aligned sequence 1
	short* m_shAligned2; // Aligned sequence 2

	// values that participate in alignment
	short* m_shMap1;
	short* m_shMap2;

	short m_MaxDiff;
	short m_MatchScore;
	short m_MismatchPenalty;
	short m_GapPenalty;

	short m_CurrentAlignment;
	struct LocalMax* m_LocalMaxArr;
	struct Trace** m_ScoreMatrix;

	short m_MatchLength;

	short m_Matches;
	short m_Mismatches;
	short m_Gaps;

	short bAligned;
};

#endif
