#ifndef AminoAcid_hpp
#define AminoAcid_hpp

#include "stl.h"


enum RESIDUE_INDEX {ARG,LYS,GLU,GLN,ASP,ASN,PRO,GLY,SER,THR,
                    HIS,ALA,ILE,LEU,VAL,PHE,CYS,MET,TYR,TRP};

/*
CLASS
  AminoAcid
   
  This class provides a general information about amino acid residues.

KEYWORD
  AminoAcid, SeqScore

AUTHORS
    Maxim Shatsky (maxshats@math.tau.ac.il)

    copyright: SAMBA Group , Tel-Aviv Univ. Israel, 1997, 1999.

CHANGES LOG

GOALS 
   AminoAcid class should help to construct user-defined classes
   that deal with amino acid residues.For example it provides one-to-one mapping
   function from character that identifies the amino acid residue to
   index which defined in range of [0,19] (see USAGE for more details),for
   this purpose class provides global enum :
   RESIDUE_INDEX {ARG,LYS,GLU,GLN,ASP,ASN,PRO,GLY,SER,
   THR,HIS,ALA,ILE,LEU,VAL,PHE,CYS,MET,TYR,TRP};
   

   AminoAcid can provide statistical information about amino acid residues,
   like "Frequency of Occurrence in Proteins". 
USAGE
 See SeqScore.
 Complexity of all functions O(1).
*/
class AminoAcid
{
private:
  static const float m_fFreqOfOccurrence[];
  static const RESIDUE_INDEX m_indexes[];
public:
  ////Returns frequency of occurrence of each amino acid residue in proteins
  //taken from M.H.Klapper,Biochem.Biophys.Res.Commun.78:1018-1024,1977 
  static float getFreqOfOccurrence(RESIDUE_INDEX index);

  ////Receives one-letter abbreviation of amino acid residue.
  //Returns index as appears in enum RESIDUE_INDEX.
  static RESIDUE_INDEX getResidueTypeIndx(const char cRes);
};

#endif








