/* $Id$ */
#ifndef _Atom_h

#define _Atom_h

#include "Particle.h"
/*
CLASS
  Atom
   
  This is a general class to hold an atom as it appears in a PDB file. 

KEYWORD
    atom, PDB, particle

AUTHORS
    Meir Fuchs (meirfux@math.tau.ac.il), Zipi Fligelman (zipo@math.tau.ac.il)

    copyright: SAMBA Group , Tel-Aviv Univ. Israel, 1997, 1999.

CHANGES LOG
<UL>
<LI>
</UL

USAGE
*/
class Atom : public Particle
{

public:
  // GROUP: Constructors

  //// An empty constructor that gives the atom members null values
  Atom();

  //// An explicit constructor
  Atom(const Vector3& p, const char ch, const unsigned short aid, 
       const unsigned short resid, 
       const char* const t,
       const char resTp);

  //// A constructor that intiate an atom type from a line of a PDB record
  explicit Atom(const char* const PDBrec);

  // GROUP: Inspectors

  //// Returns the chainId - important for working with comlexes
  char chainId() const;

  //// returns the atom index in the PDB file
  unsigned short atomIndex() const;

  //// returns the residue (thas the atom belongs to) index in the PDB file
  short residueIndex() const;

  //// Returns the atom type (non-terminated 4-char s string)
  const char *const type() const;

  //// Returns the residue type in one letter code.
  char residueType() const;

  //// Returns the residue type in 3-letter mnemonic. If no matching mnemonic
  // is found a NULL pointer is returned.
  const char *const residueName() const;

  // GROUP: Adaptors
  
  //// Sets the chain ID for various implementation like the VMD coloring
  // scheme, or RasMol web assignment. Please make sure you use it with
  // accordance to the appropriate software util.
  void setChainId(const char newChainId);
  
  //// Writing back the PBD line - without the endline 
  friend ostream& operator<<(ostream& s, const Atom& at);
  
protected:
  char  chId;                 // chain.
  unsigned short atomId;      // atom index.
  short residueId;            // residue index.
  char tp[4];                 // atom type (non-terminated 4-char string).
  char resType;
};

#endif


