#ifndef __BCOMPONENTS_H
#define __BCOMPONENTS_H

/*
CLASS
  BComponent

  A template class that inherits from Component and was written for the 
  use of the BConnectedComponents class in order to hold and handle a 
  basic group of elements using a GraphNode wrapper.  
  See Component for more details about the methods of storage.

  The BComponent template was tailored to the use of UnionList with the int
  key, and for the pre-constructing of the GraphNode objects. This is 
  neccesary because in many cases the GraphNode objects will be used from
  outside the Components for some operation on the Graph.
  This is opposed to the philosophy of the base class Component which was 
  to hide the wrapper ComponentNode from the user.

  See BConnectedComponents for more information about usage.

KEYWORDS
  ConnectedComponents, BConnectedComponents, Component, ComponentNode, GraphNode, set

AUTHORS
  Ram Nathaniel <ramn@math.tau.ac.il>

  Copyright: SAMBA group, Tel-Aviv Univ. Israel, 1998.

GOALS
  Create an easy-to-use container for a set of elements wrapped with GraphNode
  objects used to implement graph algorithms.
  The BCompnent allows fast joining, for the use of a UnionList engine. 
  This will enable easy access to the element of each union.

USAGE
  See ConnectedComponents
  
  */

#include <vector.h>
#include "UnionList.h"
#include "../gamb++/MoleculeBase.h"

#include "Connected.h"
#include "DFSGraphNode.h"

template <class elementT> class BComponent:public Component<elementT>{
public:
  //construct with only one element.
  BComponent(elementT& elem, int index){
    ComponentNode<elementT> *n=new DFSGraphNode<elementT>(&elem, index);

    head=tail=n;
    n->next=NULL;
    NumOfPairs=1;
  }

  //create BCC from a graph node.
  //since the connections between nodes have to be created seperatly, I can't
  //create the BCC from the elements like in the regular Component.
  BComponent(DFSGraphNode<elementT>* node){
    head=tail=node;
    NumOfPairs=1;
    node->next=NULL;
  }
};

#endif




