
#ifndef __COMPONENTS_H
#define __COMPONENTS_H

/*
CLASS
  Component

  A template class that was written for the use of the ConnectedComponents 
  class in order to hold and handle a basic group of elements. In order to 
  do that it uses the ComponentNode template to wrap the elements and keeps
  a link list of all the elements in a componet.

  The Component template has a few basic methods for handling the data:
  A default constructor creates an empty component.
  A constructor which creates a component with only one element.
  Add method for adding elements to a component.
  Join method which given two components, unites then :
    change both to be the union of the two components, which use the same list.
  Print method - prints the elements in the compnent.

  See ConnectedComponents for more information.

KEYWORDS
  ConnectedComponents, set

AUTHORS
  Ram Nathaniel <ramn@math.tau.ac.il>

  Copyright: SAMBA group, Tel-Aviv Univ. Israel, 1998.

GOALS
  Create an easy-to-use container for a set of elements which can handle fast
  joining, for the use of a UnionList engine. This will enable easy access to
  the element of each union.

USAGE
  See ConnectedComponents
  
  */

#include <stl.h>
#include "UnionList.h"
#include "../gamb++/MoleculeBase.h"
#include "ComponentNode.h"  



template<class elementT> class Component{
private:
  //this is private so that outside the class will not be known as Node
  //which is a common name.
  typedef ComponentNode<elementT> Node;

public: 
  //these have to be public for the join method
  int NumOfPairs;
  Node *head;
  Node *tail;

  Component(){head=NULL; tail=NULL; NumOfPairs=0;}//empty component

  //construct with only one element.
  Component(elementT& elem){
    Node *n=new Node(&elem);

    head=tail=n;
    n->next=NULL;
    NumOfPairs=1;
  }
  
  //the function converts the two objects : this and c to hold the same
  //content for we don't know yet which will the user decide to use.
  void join(Component& c){
    //this is empty
    if (head == NULL){
      head=c.head; 
      tail=c.tail;
    }

    //c is empty
    else if (c.head == NULL){}
    else{
      tail->next = c.head;
      tail = c.tail;
    }

    NumOfPairs+=c.NumOfPairs;
    c.NumOfPairs=NumOfPairs;

    c.head=head;
    c.tail=tail;
  }

  void add(elementT& elem){
    Node *n=new Node(&elem);
    n->next=head;
    head=n;
    if (tail == NULL) tail=n;
    NumOfPairs++;
  }

  void Print(){
    cout << "Number of elements is : " << NumOfPairs << endl;
    Node *n=head;
    while (n != NULL){
      cout << (Node)(*n) << endl;
      n=n->next;
    }
  }

  //retrieve elements.
  //if component is empty returns junk.
  //if element index requested is larger than the list size return the 
  //last element in the list.
  elementT& GetElement(int index){
    //if list ended return last element
    if (NumberOfPairs > index) return *(tail->element);
    Node *node=head;
    for (int i=0; i < index; i++){
      node=node->next;
    }
    return *(node->element);
  }
};

#endif

