#include "DebugStream.h"

DebugStream::DebugStream(ostream& outStream = cout) :
  //out(outStream), dbgLevel(0), withTimer(true), withNewLine(true), 
  out(&outStream), dbgLevel(0), withTimer(false), withNewLine(false), 
  withLevel(false), timer(), active(false), 
  ownStream(false), automaticFlush(false)
{}

DebugStream::DebugStream(const char *filename):
    out(&cerr), /* temporary initilization for a reference can't be blank */
    dbgLevel(0), withTimer(false), withNewLine(false), 
    withLevel(false), timer(), active(false), 
    ownStream(false), automaticFlush(false)
{
    ofstream *outstream=new ofstream(filename);
    if (!(*outstream)) cerr << "Can't open file : " 
                      << filename 
                      << " as DebugStream." 
                      << endl;
    else{
        ownStream = true;
        out=outstream;
    }
}

DebugStream::~DebugStream()
{
    if (ownStream){
        //own the output stream - has to close it.
        ((ofstream *)out)->close();
        delete out;
    }
}

void DebugStream::setDebugLevel(unsigned int debugLevel)
{
  dbgLevel = debugLevel;
}

void DebugStream::newLines(const bool print = true)
{
  withNewLine = print;
}

void DebugStream::messageLevels(const bool print = true)
{
  withLevel = print;
}

void DebugStream::messageTimes(const bool print = true)
{
  withTimer = print;
}

void DebugStream::indent(const bool print = true)
{
  withIndent = print;
}

void DebugStream::resetTimer()
{
  timer.reset();
}

void DebugStream::setFloatPrecision(unsigned int prec) 
{
    (*out).setf(ios::fixed,ios::floatfield);
    (*out).precision(prec);
}

void DebugStream::setWidth(unsigned int perc)
{
    (*out).width(perc);
}


void DebugStream::autoFlush(const bool autoflush = true)
{
    automaticFlush=autoflush;
}

DebugStream& DebugStream::operator()(unsigned int messageLevel = 0)
{
  active = (messageLevel <= dbgLevel);
  if (active) {
    if (withNewLine)
      *out << '\n';
    if (withLevel) {
      out->width(3);
      *out << messageLevel;
    }
    if (withTimer) {
      if (withLevel)
        *out << ' ';
      *out << timer << '|' << ' ';
    }
    else if (withLevel)
      *out << ':' << ' ';
    if (withIndent)
      for (unsigned int i=0; i<messageLevel/10; ++i)
        *out << ' ' << ' ';
  }
  return *this;
}

DebugStream& DebugStream::operator<<(const char* const data)
{
  if (active)
    *out << data;

  //if automaticFlush is on don't wait for the buffer flush.
  if (automaticFlush){
      ((ofstream *)out)->flush();
  }

  return *this;
}





