#ifndef __GRAPH_NODE_H
#define __GRAPH_NODE_H

#include <vector.h>
#include "ComponentNode.h"

/*
CLASS
  GraphNode

  A template class that was written for the use of the BConnectedComponents 
  class in order to wrap the basic elements and create a graph structure from 
  the elements as well as a link list under the BCompnent class objects. 
  In opposed to the base class ComponentNode this class should be public and
  open to the user for it is a good infrasturcture to implement graphs
  algorithms.

  For usage example see BConnectedComponents::DFS and DFSGraphNode.

KEYWORDS
  BConnectedComponents, BComponent, link list, set, graph

AUTHORS
  Ram Nathaniel <ramn@math.tau.ac.il>

  Copyright: SAMBA group, Tel-Aviv Univ. Israel, 1998.

GOALS
  Create a wrapper for elements to be used as vertices of a graph as well as 
  in a link list data structure. 
USAGE
  
  */


template<class elementT> 
class GraphNode:public ComponentNode<class elementT>{
public:
  //to point to an exterior database of the elements. also may be used 
  //as a key in a UnionList 
  int element_index;
  
  vector<GraphNode *> Neighbours; 
  
  //basic constructor.
  GraphNode(elementT *elem, int index):ComponentNode(elem){
    element_index=index;
  }
  
  //create an edge to a new GraphNode
  void ConnectTo(GraphNode *n){
    Neighbours.push_back(n);
  }
};


#endif




