#include "Helix.h"

typedef Helix::HelixType HelixType;

Helix:: Helix(const short helixNum,
              const int begResidue, const int endResidue, const char beginChn,
              const char endChn, const unsigned short len,
              const HelixType htype) 
  : idNum(helixNum),beginRes(begResidue), endRes(endResidue), 
    beginChain(beginChn), endChain(endChn), len(len), type(htype)
{}

Helix::Helix(const char* const PDBrec) 
  : idNum(HelixPDB::helixID(PDBrec)),
    beginRes(HelixPDB::initSeqNum(PDBrec)), 
    endRes(HelixPDB::endSeqNum(PDBrec)),
    beginChain(HelixPDB::initChainID(PDBrec)),
    endChain(HelixPDB::endChainID(PDBrec)),
    len(HelixPDB::length(PDBrec)), 
    type(HelixPDB::helixClass(PDBrec))
{}

 
Helix::Helix(const Helix& h)
    : idNum(h.idNum),beginRes(h.beginRes), endRes(h.endRes), 
      beginChain(h.beginChain), endChain(h.endChain), len(h.len), 
      type(h.type)
{}

ostream& operator<<(ostream& s, const Helix& hel)
{
  s << "HELIX " << hel.idNum << endl;
  s << "strat: (" << hel.beginRes << " , " << hel.beginChain << ")\n";
  s << "end  : (" << hel.endRes <<   " , " << hel.endChain << ")\n";
  s << "length = " << hel.len << "\tType = " ;
  switch (hel.type) {
  case HelixPDB::Unknown:
    s << "Unknown";
    break;
  case HelixPDB::RightHandedAlpha:
    s << "Right-Handed Alpha";
    break;
  case HelixPDB::ReightHandedOmega:
    s << "Right-Handed Omega";
    break;
  case HelixPDB::RightHandedPi:
    s << "Right-Handed Pi";
    break;
  case HelixPDB::RightHandedGamma:
    s << "Righ-Hnaded Gamma";
    break;
  case HelixPDB::RightHanded310:
    s << "Right-Handed 310";
    break;
  case HelixPDB::LeftHnadedAlpha:
    s << "Left-Handed Alpha";
    break;
  case HelixPDB::LeftHandedOmega:
    s << "Left-Handed Omega";
    break;
  case HelixPDB::LeftHandedGamma:
    s << "Left-Handed Gamma";
    break;
  case HelixPDB::RibbonHelix:
    s << "Ribbon Helix ";
    break;
  case HelixPDB::Polyproline:
    s << "Polyproline";
  }
  return s;
}
          
      












