#include "Atom.h"
#include "PDB.h"
#include <string>
#include <math.h>
#include "macros.h"

Atom::Atom()
  : Particle(), chId(' '), atomId(0), residueId(0), resType(' ')
{}

Atom::Atom(const Vector3& p, const char ch, const unsigned short aid, 
	   const unsigned short resid, 
	   const char* const t, const char resTp) :
  Particle(p), chId(ch), atomId(aid), residueId(resid), resType(resTp)
{
  for (int i=0; i<4; i++)
    tp[i] = t[i];
}

Atom::Atom(const char* const PDBrec) :  
  Particle(PDBrec), chId(PDB::atomChainId(PDBrec)), 
  atomId(PDB::atomIndex(PDBrec)), residueId(PDB::atomResidueIndex(PDBrec)),
  resType(PDB::atomResidueType(PDBrec))
{
  const char* const t = PDB::atomType(PDBrec);
  for (int i=0; i<4; i++)
    tp[i] = t[i];
}

char Atom::chainId() const 
{
  return chId;
}

unsigned short Atom::atomIndex() const
{
  return atomId;
}

short Atom::residueIndex() const
{
  return residueId;
}

const char *const Atom::type() const
{
  return tp;
}

char Atom::residueType() const
{
  return resType;
}

const char *const Atom::residueName() const
{
  return PDB::residueLongName(resType);
}

void Atom::setChainId(const char newChainId)
{
  chId = newChainId;
}

ostream& operator<<(ostream& s, const Atom& at)
{ 
  s << "ATOM";
  s.width(7);
  s << at.atomId;
  s << " "; 
  int i;
  for (i=0;i<4;i++)
    s << at.tp[i]; 
  s << " ";
  s.width(3);
  if (at.residueName())
    for (i=0 ; i< 3 ;++i)
      s << at.residueName()[i];
  else
    s << "   ";
  // s.width(1);
  //s.width(2);
  s << ' ' << at.chId;
  s.width(4);
  s << at.residueId;
  s.setf(ios::fixed, ios::floatfield);
  s.width(12);
  s.precision(3);
  s << at.position()[0];
  s.width(8);
  s.precision(3);
  s << at.position()[1];
  s.width(8);
  s.precision(3);
  s << at.position()[2];
  //  s << endl;
  return s;
}

  



