#ifndef _MoleculeBase_h

#define _MoleculeBase_h

#include "Vector3.h"

/*
CLASS
  MoleculeBase

  A base class for Molcule classes.
KEYWORDS
  molecule, vector, match

AUTHORS
  Meir Fuchs. (meirfux@math.tau.ac.il)

  Copyright: SAMBA group, Tel-Aviv Univ. Israel, 1997.

CHANGES LOG
<UL>
<LI>
</UL>

GOALS
  MoleculeBase hosts a virtual operator returning the vecotr positions of the 
  i'th particle in a Molecule. The class is needed in order to send Molecule
  template classes of unknown type as arguments to functions and methods
  that have to inspect particle positions. See for example 
  Match::calculateBestFit(). A final implementation can be found in the
  Molecule template class.
*/
class MoleculeBase {
public:
  //// Returns the position of the i'th particle within a Molecule. For a
  // final definition see the Molecule template class.
  virtual Vector3 operator()(unsigned int) const = 0;
};

#endif

