#include "ReferenceFrame.h"


static const char rcsid[]="$Id$";
// the empty constructor open an empty Rotation and an empty translation and zero starting place

ReferenceFrame::ReferenceFrame(): rot(), trans()
{}

// the non emty constructor 

ReferenceFrame::ReferenceFrame(const Rotation3& r, const Vector3& t) : 
  rot(r) , trans(t)
{}

ReferenceFrame::ReferenceFrame(const RigidTrans3& rt)
  : rot(rt.rotation()), trans(rt.translation())
{}

Vector3 ReferenceFrame::coordinatesInSystem(const Vector3& v) const
{
  return (!(*this))*v;
}

const Rotation3& ReferenceFrame::rotation() const
{
  return rot;
}

const Vector3& ReferenceFrame::translation() const
{
  return trans;
}

ReferenceFrame::operator RigidTrans3() const
{
  return (RigidTrans3(rot,trans));
}

Matrix3 ReferenceFrame::rotationMatrix() const
{
  return rot;
}

Vector3 ReferenceFrame::rotationAngles() const
{
  return rot.rotationAngles();
}

Vector3 operator*(const ReferenceFrame &rt, const Vector3 &v)
{
  return (rt.rot * v) + rt.trans;
}

ReferenceFrame operator*(const ReferenceFrame& rt1, const ReferenceFrame& rt2)
{
  return ReferenceFrame(rt1.rot * rt2.rot,  rt1*(rt2.trans));
}

ReferenceFrame operator!(const ReferenceFrame &rt)
{
  Rotation3 inv = !rt.rot;
  return ReferenceFrame(inv, -(inv*rt.trans));  
}

ostream& operator<<(ostream& s, const ReferenceFrame &rt)
{
  return s << rt.rotationAngles() << ' ' << rt.trans;
}

