#include <math.h>
#include "RigidTrans3.h"

static const char rcsid[] = "$Id: RigidTrans3.cc,v 1.1 1997/06/02 16:27:56 meirfux Exp $";

typedef RigidTrans3::real real;

RigidTrans3::RigidTrans3()
  : rot(1), trans()
{}

RigidTrans3::RigidTrans3(const Matrix3& r, const Vector3& t)
  : rot(Rotation3(r)), trans(t)
{}

RigidTrans3::RigidTrans3(const Rotation3& r, const Vector3& t)
  : rot(r), trans(t)
{}

RigidTrans3::RigidTrans3(const Matrix3& r, const Vector3& t, int)
  : rot(r), trans(t)
{}

RigidTrans3::RigidTrans3(const Vector3 &o, const Vector3 &u, const Vector3 &v)
{
  trans = (o+u+v)/3.0;
  Vector3 x = trans - o;
  x/= x.norm();
  Vector3 z = (v-o)&(v-u);
  z/= z.norm();
  Vector3 y = z & x;
  rot = Matrix3(x,y,z);
  rot.transpose();
}

RigidTrans3::RigidTrans3(const Vector3 &r, const Vector3 &t)
  : rot(r[0], r[1], r[2]) , trans(t)
{}

const Matrix3& RigidTrans3::rotation() const
{
  return rot;
}

Vector3 RigidTrans3::rotationAngles() const
{
  return rot.rot();
}

const Vector3& RigidTrans3::translation() const
{
  return trans;
}

Vector3 operator*(const RigidTrans3 &rt, const Vector3 &v)
{
  return (rt.rot * v) + rt.trans;
}

RigidTrans3 operator*(const RigidTrans3 &rt1, const RigidTrans3 &rt2)
{
  return RigidTrans3(rt1.rot * rt2.rot,  rt1*(rt2.trans), 0);
}

RigidTrans3 operator!(const RigidTrans3 &rt)
{
  Matrix3 inv = rt.rot.transposeMatrix();
  return RigidTrans3(inv, -(inv*(rt.trans)), 0);  
}

ostream& operator<<(ostream& s, const RigidTrans3 &rt)
{
  return s << rt.rotationAngles() << ' ' << rt.trans;
}

istream& operator>>(istream& s, RigidTrans3& rt)
{
  Vector3 rotvec;
  s >> rotvec >> rt.trans;
  rt.rot=Matrix3(rotvec[0], rotvec[1], rotvec[2]);
  return s;
}
  


