#include "SecondStruct.h"
#include <assert.h>

typedef SecondStruct::Helices Helices;
typedef SecondStruct::Sheets Sheets;
typedef SecondStruct::SSType SSType;

SecondStruct::SecondStruct() 
  : alpha(), beta()
{}

SecondStruct::SecondStruct(const char* const PDBFileName)
  : alpha(), beta()
{
  ifstream PDBFileStream(PDBFileName);
  if (!PDBFileStream) {
      cerr << "couldn't open file " << PDBFileName << endl;
      exit(1);
  }
  readSSFromPDBFile(PDBFileStream);
  PDBFileStream.close();  
}

SecondStruct::SecondStruct(const SecondStruct& ss)
{
    if (alpha.size() != 0)
        delete &alpha;
    if (beta.size() != 0)
        delete &beta;
    alpha = ss.alpha;
    beta = ss.beta;
    /* for(Helices::iterator it = const_cast<Helix *>(ss.alpha.begin());
        it != ss.alpha.end(); ++it)
        alpha.push_back(Helix(*it));
    for (Sheets::iterator jt = const_cast<Sheet *>(ss.beta.begin());
         jt !=ss.beta.end(); ++jt)
         beta.push_back(Sheet(*jt));*/
}

SecondStruct& SecondStruct::operator=(const SecondStruct& rhs)
{
    if (&rhs != this) {
        if (alpha.size() != 0)
          delete &alpha;
        if (beta.size() != 0)
          delete &beta;
        for (Helices::iterator it = const_cast<Helix *>(rhs.alpha.begin()); 
             it!= rhs.alpha.end(); ++it)
        alpha.push_back(*it);
        for (Sheets::iterator it = const_cast<Sheet *>(rhs.beta.begin());
             it != rhs.beta.end(); ++it)
        beta.push_back(*it);
        assert(alpha.size() == rhs.alpha.size());
        assert(beta.size() == rhs.beta.size());
    }
    return *this;
}

Helix& SecondStruct::operator[](const int helixId) 
{
    return alpha[helixId-1];
}

Sheet& SecondStruct::operator[](const char sheetId)
{
    char res[3];
    res[0]=res[2]=' '; res[1]=sheetId;
    for (Sheets::iterator it = beta.begin(); it != beta.end(); ++it)
        if ((*it).isCurrSheet(res))
            return (*it);
    return beta[beta.size()];
}

void SecondStruct::addAHelix(const Helix& helix)
{
    alpha.push_back(helix);
}
void SecondStruct::addBSheet(const Sheet& sheet)
{
    beta.push_back(sheet);
}


bool SecondStruct::isResInAHelix(const short helixNum, const int resNum) const
{
  return (resNum >= alpha[helixNum-1].startResidue() &&
          resNum <= alpha[helixNum-1].endResidue());
}

// might have a problem here since I keep the id in three letters space 
// to think what can be done about it
bool SecondStruct::isInBSheet(const char sheetId[], const int resNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if ((*it).isCurrSheet(sheetId))
      for (Sheet::iterator jt = (*it).begin(); jt != (*it).end() ; ++jt)
          if ((*jt).isResInStrand(resNum)) 
              return true;
  return false;
}

SSType SecondStruct::residueInSS(const int atomNum) const
{
  for (Helices::iterator it = const_cast<Helix*>(alpha.begin());
       it != alpha.end(); ++it)
      if (isResInAHelix(it-alpha.begin()+1, atomNum))    
          return AHelix;
  
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) {
      if (isInBSheet(it->sheetId(), atomNum))
          return BSheet;
  }
  return Unknown;
}

bool SecondStruct::isFromSameSS(const vector<short>& indices)
{
    //typedef SecondStruct::SSType SSType;
    int len = indices.size();
    bool same = true;
    SSType type[len];

     for (int i=0; i < len ; ++i) {
         type[i] = residueInSS(indices[i]);
	 if  (type[i] == Unknown)
		return false;
    }
    for (int i=0; i < len ; ++i) 
         for (int j=i; j < len ; ++j) 
             if (type[i]!=type[j])
                 return false;
     
     if (type[0] == AHelix) {
         short helicesNum[len];
         for (int i=0; i < len ; ++i)
             helicesNum[i]=numOfAHelix(indices[i]);
         for (int i=0; i < len ; ++i) 
             for (int j=i; j < len; ++j) 
                 if (helicesNum[i]!=helicesNum[j])
                     return false;
     }
     else if (type[0] == SecondStruct::BSheet) {
         pair<const char *,short> sheetStrandId[len];     
         for (int i=0; i < len; ++i)
             sheetStrandId[i]  = sheetAndStrandId(indices[i]);
         for (int i=0; i < len ; ++i) 
             for (int j=i; j < len; ++j) {
                 int k = strncmp(sheetStrandId[i].first,
                                 sheetStrandId[j].first,3);
                 if ((k != 0) || 
                     ((k == 0) && 
                      (sheetStrandId[i].second != sheetStrandId[j].second)))
                     return false;
             }
     }
     return same;
}

bool SecondStruct::isFromSameSS(const short indices[], int len)
{
    bool same = true;
    SSType type[len];

    for (int i=0; i < len ; ++i) {
        type[i] = residueInSS(indices[i]);
        if  (type[i] == Unknown)
            return false;
    }
    for (int i=0; i < len ; ++i) 
        for (int j=i; j < len ; ++j) 
            if (type[i]!=type[j])
                return false;
    
    if (type[0] == AHelix) {
        short helicesNum[len];
        for (int i=0; i < len ; ++i)
            helicesNum[i]=numOfAHelix(indices[i]);
        for (int i=0; i < len ; ++i) 
            for (int j=i; j < len; ++j) 
                if (helicesNum[i]!=helicesNum[j])
                    return false;
    }
    else if (type[0] == SecondStruct::BSheet) {
        pair<const char *,short> sheetStrandId[len];     
        for (int i=0; i < len; ++i)
            sheetStrandId[i]  = sheetAndStrandId(indices[i]);
        for (int i=0; i < len ; ++i) 
            for (int j=i; j < len; ++j) {
                int k = strncmp(sheetStrandId[i].first,
                                sheetStrandId[j].first,3);
                if ( (k != 0) ||
                     ((k == 0) &&
                      (sheetStrandId[i].second != sheetStrandId[j].second)))
                    return false;
            }
    }
    return same;
}

short SecondStruct::numOfAHelix(const int resNum) const
{
  for (Helices::iterator it = const_cast<Helix*>(alpha.begin());
       it != alpha.end(); ++it)
      if ((*it).isResidueInHelix(resNum))
          //if (isInAHelix(it-alpha.begin()+1, resNum))
          return (it-alpha.begin()+1);
  return -1;
}

const char * SecondStruct::numOfBSheet(const int resNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if (isInBSheet(it->sheetId(), resNum))
      return (it->sheetId());
  return '\0';
}


pair<const char *,short> SecondStruct::sheetAndStrandId(const int resNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    for (Sheet::iterator jt = (*it).begin(); jt != (*it).end(); ++jt) 
        if ((*jt).isResInStrand(resNum)) {
            return pair<const char *,short>(it->sheetId(), jt->strandNumber());
        }
  return pair<const char *,short>((const char *)NULL,-1);
}
 
void SecondStruct::readSSFromPDBFile(ifstream& PDBFileStream)
{
  char line[85];
  int j=0;
  while (PDBFileStream.get(line,85,'\n')) {
    if (HelixPDB::isHelix(line)) {
      alpha.push_back(Helix(line));
    }
    else if (StrandPDB::isStrand(line)) {
        if (StrandPDB::sense(line)==0 || StrandPDB::currAtom(line) == ' ') {
          // first strand
          int max_size = StrandPDB::numOfStrandsInSheet(line); 
            Sheet bs(line);
            for (int i=1; i < max_size; ++i) {
                PDBFileStream.get(*line);
                PDBFileStream.get(line,85,'\n');
                assert(StrandPDB::isStrand(line));
                bs.add(line);
                
            }
            beta.push_back(bs);
            ++j;
        }
    }
    else if (PDB::isATOMrec(line)) /* if we have reached
                                      the atoms no change there 
                                      will be any information on the 
                                      helices afterwards */
        break;
    PDBFileStream.get(*line);
  }
}


ostream& operator<<(ostream& s, const SecondStruct& ss)
{
  for (Helices::iterator it = const_cast<Helix*>(ss.alpha.begin());
       it != ss.alpha.end(); ++it)
    s << *it << endl;
  for (Sheets::iterator it = const_cast<Sheet*>(ss.beta.begin());
       it != ss.beta.end(); ++it) {
      s << *it << endl;
  }
  return s;
}
  
