#include "SecondStruct.h"
#include <assert.h>

typedef SecondStruct::Helices Helices;
typedef SecondStruct::Sheets Sheets;
typedef SecondStruct::SSType SSType;

SecondStruct::SecondStruct() 
  : alpha(), beta()
{}

SecondStruct::SecondStruct(const char* const PDBFileName)
  : alpha(), beta()
{
  ifstream PDBFileStream(PDBFileName);
  if (!PDBFileStream) {
      cerr << "couldn't open file " << PDBFileName << endl;
      exit(1);
  }
  readSSFromPDBFile(PDBFileStream);
  PDBFileStream.close();  
}

SecondStruct::SecondStruct(const SecondStruct& ss)
{
    if (alpha.size() != 0)
        delete &alpha;
    if (beta.size() != 0)
        delete &beta;
    alpha = ss.alpha;
    beta = ss.beta;
    /* for(Helices::iterator it = const_cast<Helix *>(ss.alpha.begin());
        it != ss.alpha.end(); ++it)
        alpha.push_back(Helix(*it));
    for (Sheets::iterator jt = const_cast<Sheet *>(ss.beta.begin());
         jt !=ss.beta.end(); ++jt)
         beta.push_back(Sheet(*jt));*/
}

SecondStruct& SecondStruct::operator=(const SecondStruct& rhs)
{
    if (&rhs != this) {
        if (alpha.size() != 0)
          delete &alpha;
        if (beta.size() != 0)
          delete &beta;
        for (Helices::iterator it = const_cast<Helix *>(rhs.alpha.begin()); 
             it!= rhs.alpha.end(); ++it)
        alpha.push_back(*it);
        for (Sheets::iterator it = const_cast<Sheet *>(rhs.beta.begin());
             it != rhs.beta.end(); ++it)
        beta.push_back(*it);
        assert(alpha.size() == rhs.alpha.size());
        assert(beta.size() == rhs.beta.size());
    }
    return *this;
}

Helix& SecondStruct::operator[](const int helixId) 
{
    return alpha[helixId-1];
}

Sheet& SecondStruct::operator[](const char strandId)
{
    char res[3];
    res[0]=res[2]=' '; res[1]=strandId;
    for (Sheets::iterator it = beta.begin(); it != beta.end(); ++it)
        if ((*it).isCurrSheet(res))
            return (*it);
    return beta[beta.size()];
}

bool SecondStruct::isInAHelix(const short helixNum, const int atomNum) const
{
  return (atomNum+1 >= alpha[helixNum-1].startResidue() &&
          atomNum+1 <= alpha[helixNum-1].endResidue());
}

// might have a problem here since I keep the id in three letters space 
// to think what can be done about it
bool SecondStruct::isInBSheet(const char sheetId, const int atomNum) const
{
  char name[3];
  name[0]=name[2]=' '; 
  name[1] = sheetId; 
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if ((*it).isCurrSheet(name))
      for (Sheet::iterator jt = (*it).begin(); jt != (*it).end() ; ++jt)
        if (atomNum-1 >= jt->initResidueNum() &&
            atomNum-1 <= jt->endResidueNum())
          return true;
  return false;
}

SSType SecondStruct::atomInSS(const int atomNum) const
{
  for (Helices::iterator it = const_cast<Helix*>(alpha.begin());
       it != alpha.end(); ++it)
    if (isInAHelix(it-alpha.begin()+1, atomNum))
      return AHelix;
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if (isInBSheet(it->sheetId(), atomNum))
      return BSheet;
  return Unknown;
}

bool SecondStruct::isFromSameSS(const vector<short>& indices)
{
  int len = indices.size();
  short where_a[len];
  pair<char,short> where_b[len];
  for (int i=0; i< len; ++i) {
    where_a[i]=-1;
    where_b[i]=pair<char,short>('?',-1);
  }
  for (vector<short>::iterator it= const_cast<short *>(indices.begin()); 
       it!=indices.end(); ++it) {
    int offset = it-indices.begin();
    where_a[offset]=numOfAHelix(*it);
    if (where_a[offset] > -1 && offset > 0) {
      for (int j=0; j < offset; ++j)
        if (where_a[j]==where_a[offset])
          return false;
    }
    else {
      where_b[offset] = sheetAndStrandId(*it);
      if (where_b[offset].first!='\0' && offset > 0)
        for (int j=0; j < offset; ++j)
          if (where_b[j]==where_b[offset])
            return false;
    }
  }
  return true;
}
bool SecondStruct::isFromSameSS(const short indices[], int len)
{
  short where_a[len];
  pair<char,short> where_b[len];
  for (int i=0; i< len; ++i) {
    where_a[i]=-1;
    where_b[i]=pair<char,short>('?',-1);
  }
  for (int i=0; i < len ; ++i) {
    where_a[i]=numOfAHelix(indices[i]);
    if (where_a[i] > -1 && i > 0) {
      for (int j=0; j < i; ++j)
        if (where_a[j]==where_a[i])
          return false;
    }
    else {
      where_b[i] = sheetAndStrandId(indices[i]);
      if (where_b[i].first!='\0' && i > 0)
        for (int j=0; j < i; ++j)
          if (where_b[j]==where_b[i])
            return false;
    }
  }
  return true;
}
short SecondStruct::numOfAHelix(const int atomNum) const
{
  for (Helices::iterator it = const_cast<Helix*>(alpha.begin());
       it != alpha.end(); ++it)
    if (isInAHelix(it-alpha.begin()+1, atomNum))
      return (it-alpha.begin()+1);
  return -1;
}

char SecondStruct::numOfBSheet(const int atomNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if (isInBSheet(it->sheetId(), atomNum))
      return (it->sheetId());
  return '\0';
}


pair<char,short> SecondStruct::sheetAndStrandId(const int atomNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    for (Sheet::iterator jt = (*it).begin(); jt != (*it).end(); ++jt) 
      if (((atomNum-1) >= jt->initResidueNum()) &&
          ((atomNum-1) <= jt->endResidueNum())) 
        return pair<char,short>(it->sheetId(), jt->strandNumber());
  return pair<char,short>('\0',-1);
}
 
void SecondStruct::readSSFromPDBFile(ifstream& PDBFileStream)
{
  char line[85];
  while (PDBFileStream.get(line,85,'\n')) {
    if (HelixPDB::isHelix(line)) {
      alpha.push_back(Helix(line));
    }
    else if (StrandPDB::isStrand(line)) {
        if (StrandPDB::sense(line)==0 || StrandPDB::currAtom(line) == ' ') {
          // first strand
          int max_size = StrandPDB::numOfStrandsInSheet(line); 
            Sheet bs(line);
            for (int i=1; i < max_size; ++i) {
                PDBFileStream.get(*line);
                PDBFileStream.get(line,85,'\n');
                //     cerr << line << endl;
                assert(StrandPDB::isStrand(line));
                bs.add(line);
                // if (i== bs.max_size()-1) {
                //  cerr << "last line:\n" << line << endl;
                // }
            }
            cerr << "length sheet " << bs.size() << endl;
            beta.push_back(bs);
        }
    }
    else if (PDB::isATOMrec(line)) /* if we have reached
                                      the atoms no change there 
                                      will be any information on the 
                                      helices afterwards */
        break;
    PDBFileStream.get(*line);
  }
}


ostream& operator<<(ostream& s, const SecondStruct& ss)
{
  for (Helices::iterator it = const_cast<Helix*>(ss.alpha.begin());
       it != ss.alpha.end(); ++it)
    s << *it << endl;
  for (Sheets::iterator it = const_cast<Sheet*>(ss.beta.begin());
       it != ss.beta.end(); ++it)
    s << *it << endl;
  return s;
}
  
