#include "StrandPDB.h"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include <iostream>

typedef StrandPDB::RelativeParallelism RelativeParallelism;

bool StrandPDB::isStrand(const char* const PDBrec)
{
  return (strncmp(PDBrec,"SHEET",5)==0);
}

short StrandPDB::strandNum(const char* const PDBrec)
{
  return atoi(PDBrec+strandNumberField);
}
void StrandPDB::strandSheetId(const char* const PDBrec, char id[])
{
    
   strncpy(id, PDBrec+sheetIDField-1,3);
}

short StrandPDB::numOfStrandsInSheet(const char* const PDBrec)
{
  return atoi(PDBrec+numStrandsField);
}
void StrandPDB::initResName(const char* const PDBrec, char res[])
{
  strncpy(res,PDBrec+initResNameField,3);
  res[3]='\0';
}
char StrandPDB::initChainId(const char* const PDBrec)
{
  return PDBrec[initChainIdField-1];
}
int StrandPDB::initSeqNum(const char* const PDBrec)
{
  return atoi(PDBrec+initSeqNumField);
}
char StrandPDB::initICode(const char* const PDBrec)
{
  return PDBrec[initICodeField-1];
}

void StrandPDB::endResName(const char* const PDBrec,char res[])
{
  strncpy(res, PDBrec+endResNameField,3);
  res[4]='\0';
}

char StrandPDB::endChainId(const char* const PDBrec)
{
  return PDBrec[endChainIdField-1];
}
int StrandPDB::endSeqNum(const char* const PDBrec)
{
  return atoi(PDBrec+endSeqNumField);
}
char StrandPDB::endICode(const char* const PDBrec)
{
  return PDBrec[endICodeField-1];
}
RelativeParallelism StrandPDB::sense(const char* const PDBrec)
{
  if (PDBrec[senseField-1] == '-' && PDBrec[senseField]=='1')
    return AntiParallel;
  else if (PDBrec[senseField] == '1')
    return Parallel;
  else  /* might need to add here info 
           regarding the REMARK records */
    return FirstStrand;
}
char StrandPDB::currAtom(const char* const PDBrec)
{
  // might be in a different level like the strand 
  // and sheet level
  
  // removing the assert and changing it to a warning
  //assert(PDBrec[curAtomField]=='N' || PDBrec[curAtomField]=='O');
  if (!(PDBrec[curAtomField]=='N' || PDBrec[curAtomField]=='O')) 
    cerr << "might be wrong data in line: " << PDBrec << endl
         << "atoms of connection must be either N or O\n";
  
  return PDBrec[curAtomField];
}
void StrandPDB::currResName(const char* const PDBrec, char res[])
{
  strncpy(res, PDBrec+curResNameField, 3);
  res[3]='\0';
}

char StrandPDB::currChainId(const char* const PDBrec)
{
  return PDBrec[curChainIdField];
}
int StrandPDB::currResSeq(const char* const PDBrec)
{
  return atoi(PDBrec+curResSeqField);
}
char StrandPDB::currICode(const char* const PDBrec)
{
  return PDBrec[curICodeField];
}
char StrandPDB::prevAtom(const char* const PDBrec)
{
  // same notice as above
  // assert(PDBrec[prevAtomField]=='N' || PDBrec[prevAtomField]=='O');
  if (!(PDBrec[curAtomField]=='N' || PDBrec[curAtomField]=='O')) 
    cerr << "might be wrong data in line: " << PDBrec << endl
         << "atoms of connection must be either N or O\n";
  return PDBrec[prevAtomField];
}

void StrandPDB::prevResName(const char* const PDBrec, char res[])
{
  strncpy(res, PDBrec+prevResNameField,3);
  res[3]='\0';
}

char StrandPDB::prevChainId(const char* const PDBrec)
{
  return PDBrec[prevChainIdField];
}

int StrandPDB::prevResSeq(const char* const PDBrec)
{
  return atoi(PDBrec+prevResSeqField);
}

char StrandPDB::prevICode(const char* const PDBrec)
{
  return PDBrec[prevICodeField];
}











