#include "Vector2.h"
#include "macros.h"
#include "math.h"

typedef Vector2::real real;

Vector2::Vector2()
{
  c[0]=c[1]=0.0;
}

Vector2::Vector2(const real& nx, const real& ny)
{
  c[0]=nx;
  c[1]=ny;
}
 
Vector2::Vector2(const real x[2])
{
  c[0]=x[0];
  c[1]=x[1];
}



real Vector2::operator[](const unsigned short coord) const
{
  if (coord < 2)
    return (c[coord]);
  else
    throw "wrong index index should be either zero or one";
}

real Vector2::norm() const 
{
  return (sqrt(sqr(c[0])+sqr(c[1])));
}


real Vector2::norm2() const
{
  return (sqr(c[0])+sqr(c[1]));
}

real Vector2::dist(const Vector2& p) const 
{
  return (sqrt(sqr(c[0]-p.c[0])+sqr(c[1]-p.c[1])));
}

real Vector2::dist2(const Vector2& p) const
{
  return (sqr(c[0]-p.c[0])+sqr(c[1]-p.c[1]));
}

Vector2 operator+(const Vector2& p1, const Vector2& p2)
{
  return Vector2(p1.c[0]+p2.c[0], p1.c[1]+p2.c[1]);
}

Vector2 operator-(const Vector2& p1, const Vector2& p2)
{
  return Vector2(p1.c[0]-p2.c[0], p1.c[1]-p2.c[1]);
}

Vector2 operator-(const Vector2& p)
{
  return Vector2(0.0-p.c[0], 0.0-p.c[1]);
}

real operator*(const Vector2& p1, const Vector2& p2)
{
  return (p1.c[0]*p2.c[0]+p2.c[1]*p2.c[1]);
}

Vector2 operator*(const Vector2& p, const real& m)
{
  return Vector2(p.c[0]*m, p.c[1]*m);
}

Vector2 operator*(const real& m, const Vector2& p)
{
  return Vector2(m*p.c[0], m*p.c[1]);
}

Vector2 operator/(const Vector2& p, const real& m)
{
  return Vector2(p.c[0]/m, p.c[1]/m);
}


float operator|(const Vector2& p1, const Vector2& p2)
{
  return sqrt(sqr(p1.c[0]-p2.c[0])+sqr(p1.c[1]-p2.c[1]));
}

real operator^(const Vector2& p1, const Vector2& p2)
{
  return acos((p1.c[0]*p2.c[0]+p1.c[1]*p2.c[1])/
              (sqrt(sqr(p1.c[0])+sqr(p1.c[1])*
                    sqrt(sqr(p2.c[0])+sqr(p2.c[1])))));
}

Vector2& Vector2::operator+=(const Vector2 &p)
{
  c[0]+=p.c[0]; c[1]+=p.c[1];
  return *this;
}


 
Vector2& Vector2::operator-=(const Vector2 &p)
{
  c[0]-=p.c[0]; c[1]-=p.c[1];
  return *this;
}

 
Vector2& Vector2::operator*=(const real &m)
{
  c[0]*=m; c[1]*=m;
  return *this;
}


Vector2& Vector2::operator/=(const real &m)
{
  c[0]/=m; c[1]/=m;
  return *this;
}

ostream& operator<<(ostream& s, const Vector2 &v)
{
  s << "(" << v.c[0] << "," <<v.c[1]<< ")" << endl;
  return s;
}
  




