#include "kts.h"
#include "MolWithSes.h"
#include <assert.h>
#include <stdio.h>

kts::kts() 
{
    for (int i=0; i < 5; ++i)
        ktuple[i] = -1;
}

kts::kts(const int i1, const int i2, const int i3,
         const int i4, const int i5)
{
    ktuple[0]=i1;
    ktuple[1]=i2;
    ktuple[2]=i3;
    ktuple[3]=i4;
    ktuple[4]=i5;
}

bool kts::fromSameSes(const MolWithSes& molses)
{
    typedef SecondStruct::SSType SSType;
    // typedef SecondStruct::Unknown Unknown;
    // typedef SecondStruct::AHelix AHelix;
    // typedef SecondStruct::BSheet BSheet;

    SSType type[5];
    short helicesNum[5];
    pair<char,short> sheetStrandId[5];
    bool same=true;

    for (int i=0; i < 5 ; ++i)
        type[i] = molses.atomInSS(ktuple[i]);
    for (int i=0; i < 5 && same ; ++i) 
        for (int j=i; j < 5 && same ; ++j) 
            if (type[i]!=type[j])
                same = false;
    if (same) {
        if (type[0]==SecondStruct::AHelix) {
            for (int i=0; i <5; ++i)
                helicesNum[i] = molses.numOfAHelix(ktuple[i]);
            bool same=true;
            for (int i=0; i < 5 && same ; ++i) 
                for (int j=i; j < 5 && same ; ++j) 
                    if (helicesNum[i]!=helicesNum[j])
                        same = false;
            return same;
        }
        else if (type[0] == SecondStruct::BSheet) {
            for (int i=0; i <5; ++i)
               sheetStrandId[i] = molses.sheetAndStrandId(ktuple[i]);
            bool same=true;
            for (int i=0; i < 5 && same ; ++i) 
                for (int j=i; j < 5 && same ; ++j) 
                    if ((sheetStrandId[i].first != sheetStrandId[j].first) ||
                        (sheetStrandId[i].first == sheetStrandId[j].first &&
                         sheetStrandId[i].second != sheetStrandId[j].second))
                        same = false;
            return same;

        }
        else
            return false;
    }
    else
        return false;
}


istream& operator>>(istream& is, kts& new_kts)
{
    char comma[4];
    char lb,rb;
    int i1,i2,i3,i4,i5;
    
    is >> lb;
    if (lb == '(') {
        is >> i1 >> comma[0] >> i2 >> comma[1] >> i3 >> comma[2]
           >> i4 >> comma[3] >> i5 >> rb;
    
        if (rb != ')') is.clear(ios::badbit);
    }
    else {
        is.putback(lb);
        return is;
    }
    //cerr << lb << i1 << comma[0] << i2 << comma[1] << i3 << comma[2]
    //    << i4 << comma[3] << i5 << rb << endl;
    if (is) new_kts = kts(i1,i2,i3,i4,i5);
    return is;
}
    

ostream& operator<<(ostream& os, const kts& out_kts) 
{
    os << "( " ;
    for (int i=0; i < 4; ++i)
        os << out_kts.ktuple[i] << " , ";
    os << out_kts.ktuple[4] << " )";
    return os;
}
