#include <stdlib.h>
#include "PDB.h"

static const char rcsid[] = "$Id: PDB.cc,v 1.1 1997/06/02 16:20:31 meirfux Exp $";

static const unsigned short res = 23;
static const char residues[] =    
  "ALAA ARGR ASPD ASNN CYSC GLNQ GLUE GLYG HISH ILEI LEUL LYSK METM PHEF PROP SERS THRT TYRY TRPW VALV AS B GL Z UNKX";

bool PDB::isATOMrec(const char *const PDBrec)
{
  return (PDBrec[0]=='A' && PDBrec[1]=='T' && 
	  PDBrec[2]=='O' && PDBrec[3]=='M');
}

bool PDB::isHETATMrec(const char *const PDBrec)
{
  return (PDBrec[0]=='H' && PDBrec[1]=='E' && PDBrec[2]=='T' &&
	  PDBrec[3]=='A' && PDBrec[4]=='T' && PDBrec[5]=='M');
}

bool PDB::isCaATOMrec(const char *const PDBrec)
{
  return (isATOMrec(PDBrec) && PDBrec[13] == 'C' && PDBrec[14] == 'A');
}

char PDB::residueShortName(const char *const resName)
{
  for (int i=0; i<res*5; i+=5)
    if (resName[0] == residues[i] && resName[1] == residues[i+1] &&
	resName[2] == residues[i+2])
      return residues[i+3];
  return 'X';
}

const char * const PDB::residueLongName(const char resChar)
{
  for (int i=3; i<res*5; i+=5) 
    if (resChar == residues[i]) 
      return residues+(i-3); 
  return 0;
}

float PDB::atomXCoord(const char *const PDBrec)
{
  return atof(PDBrec+atomXCoordField); 
}

float PDB::atomYCoord(const char *const PDBrec)
{ 
  return atof(PDBrec+atomYCoordField);
}

float PDB::atomZCoord(const char *const PDBrec)
{ 
  return atof(PDBrec+atomZCoordField); 
}

short PDB::atomIndex(const char *const PDBrec)
{ 
  return atoi(PDBrec+atomIndexField);
}

char  PDB::atomChainId(const char *const PDBrec)
{ 
  return PDBrec[atomChainIdField];
}

short PDB::atomResidueIndex(const char *const PDBrec)
{ 
  return atoi(PDBrec+atomResIndexField); 
}

char  PDB::atomResidueType(const char *const PDBrec)
{
  return residueShortName(PDBrec+atomResTypeField);
}

const char *PDB::atomType(const char *const PDBrec)
{
  return PDBrec+atomTypeField;
}




