#include "SecondStruct.h"
#include <assert.h>

typedef SecondStruct::Helices Helices;
typedef SecondStruct::Sheets Sheets;
typedef SecondStruct::SSType SSType;


  const  unsigned short SecondStruct::DSSP_MAX_LENGTH_OF_RECORD = 140;
  const  short SecondStruct::DSSP_START_OF_PDB_RESIDUE_FIELD = 5;
  const  short SecondStruct::DSSP_SS_TYPE_FIELD = 16;
  const  short SecondStruct::DSSP_CHAIN_FIELD = 11;
  
  const  char SecondStruct::DSSP_HELIX_SS_TYPE = 'H';
  const  char SecondStruct::DSSP_STRAND_SS_TYPE = 'E';

  const  unsigned short PDB_MAX_LENGTH_OF_RECORD = 85;

SecondStruct::SecondStruct() 
  : alpha(), beta()
{}

SecondStruct::SecondStruct(const char* const PDBFileName)
  : alpha(), beta()
{
  ifstream PDBFileStream(PDBFileName);
  if (!PDBFileStream) {
      cerr << "couldn't open file " << PDBFileName << endl;
      exit(1);
  }
  readSSFromPDBFile(PDBFileStream);
  PDBFileStream.close();  
}

SecondStruct::SecondStruct(const SecondStruct& ss)
{
    if (alpha.size() != 0)
        delete &alpha;
    if (beta.size() != 0)
        delete &beta;
    alpha = ss.alpha;
    beta = ss.beta;
    /* for(Helices::iterator it = const_cast<Helix *>(ss.alpha.begin());
        it != ss.alpha.end(); ++it)
        alpha.push_back(Helix(*it));
    for (Sheets::iterator jt = const_cast<Sheet *>(ss.beta.begin());
         jt !=ss.beta.end(); ++jt)
         beta.push_back(Sheet(*jt));*/
}

SecondStruct& SecondStruct::operator=(const SecondStruct& rhs)
{
    if (&rhs != this) {
        if (alpha.size() != 0)
          delete &alpha;
        if (beta.size() != 0)
          delete &beta;

        for (Helices::iterator it = const_cast<Helix *>(rhs.alpha.begin()); 
             it!= rhs.alpha.end(); ++it)
        alpha.push_back(*it);

        for (Sheets::iterator it2 = const_cast<Sheet *>(rhs.beta.begin());
             it2 != rhs.beta.end(); ++it2)
        beta.push_back(*it2);

        assert(alpha.size() == rhs.alpha.size());
        assert(beta.size() == rhs.beta.size());
    }
    return *this;
}

Helix& SecondStruct::operator[](const int helixId) 
{
    return alpha[helixId-1];
}

Sheet& SecondStruct::operator[](const char sheetId)
{
    char res[3];
    res[0]=res[2]=' '; res[1]=sheetId;
    for (Sheets::iterator it = beta.begin(); it != beta.end(); ++it)
        if ((*it).isCurrSheet(res))
            return (*it);
    return beta[beta.size()];
}

void SecondStruct::addAHelix(const Helix& helix)
{
    alpha.push_back(helix);
}
void SecondStruct::addBSheet(const Sheet& sheet)
{
    beta.push_back(sheet);
}


bool SecondStruct::isResInAHelix(const short helixNum, const int resNum) const
{
  return (resNum >= alpha[helixNum-1].startResidue() &&
          resNum <= alpha[helixNum-1].endResidue());
}

// might have a problem here since I keep the id in three letters space 
// to think what can be done about it
bool SecondStruct::isInBSheet(const char sheetId[], const int resNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if ((*it).isCurrSheet(sheetId))
      for (Sheet::iterator jt = (*it).begin(); jt != (*it).end() ; ++jt)
          if ((*jt).isResInStrand(resNum)) 
              return true;
  return false;
}

SSType SecondStruct::residueInSS(const int atomNum) const
{
  for (Helices::iterator it = const_cast<Helix*>(alpha.begin());
       it != alpha.end(); ++it)
      if (isResInAHelix(it-alpha.begin()+1, atomNum))    
          return AHelix;
  
  for (Sheets::iterator it2 = const_cast<Sheet*>(beta.begin()); 
       it2 != beta.end(); ++it2) {
      if (isInBSheet(it2->sheetId(), atomNum))
          return BSheet;
  }
  return Unknown;
}

bool SecondStruct::isFromSameSS(const vector<short>& indices)
{
    //typedef SecondStruct::SSType SSType;
    int len = indices.size();
    bool same = true;

    SSType* type_ = new SSType[len];

     for (int i1=0; i1 < len ; ++i1) {
         type_[i1] = residueInSS(indices[i1]);
	 if  (type_[i1] == Unknown)
	 {
		delete type_;
		return false;
	 }
    }
    for (int i=0; i < len ; ++i) 
         for (int j=i; j < len ; ++j) 
             if (type_[i]!=type_[j])
				 {
					delete type_;
					return false;
			 }
     
     if (type_[0] == AHelix) {
         short* helicesNum= new short[len];
         for (int i1=0; i1 < len ; ++i1)
             helicesNum[i]=numOfAHelix(indices[i1]);
         for (int i=0; i < len ; ++i) 
             for (int j=i; j < len; ++j) 
                 if (helicesNum[i]!=helicesNum[j])
					 {
						delete type_;
						return false;
				 }
		 delete  helicesNum;
     }
     else if (type_[0] == SecondStruct::BSheet) {
         pair<const char *,short>* sheetStrandId = new pair<const char *,short>[len];     
         for (int i1=0; i1 < len; ++i1)
             sheetStrandId[i1]  = sheetAndStrandId(indices[i1]);
		 // int a[size];

         for (int i=0; i < len ; ++i) 
             for (int j=i; j < len; ++j) {
                 int k = strncmp(sheetStrandId[i].first,
                                 sheetStrandId[j].first,3);
                 if ((k != 0) || 
                     ((k == 0) && 
                      (sheetStrandId[i].second != sheetStrandId[j].second)))
					{
						delete type_;
						 return false;
					}
             }
     delete sheetStrandId;
	 }
	 {
 	 delete type_;
     return same;
}

bool SecondStruct::isFromSameSS(const short indices[], int len)
{
    bool same = true;
    SSType type[len];

    for (int i=0; i < len ; ++i) {
        type[i] = residueInSS(indices[i]);
        if  (type[i] == Unknown)
            return false;
    }
    for (int i=0; i < len ; ++i) 
        for (int j=i; j < len ; ++j) 
            if (type[i]!=type[j])
                return false;
    
    if (type[0] == AHelix) {
        short helicesNum[len];
        for (int i=0; i < len ; ++i)
            helicesNum[i]=numOfAHelix(indices[i]);
        for (int i=0; i < len ; ++i) 
            for (int j=i; j < len; ++j) 
                if (helicesNum[i]!=helicesNum[j])
                    return false;
    }
    else if (type[0] == SecondStruct::BSheet) {
        pair<const char *,short> sheetStrandId[len];     
        for (int i=0; i < len; ++i)
            sheetStrandId[i]  = sheetAndStrandId(indices[i]);
        for (int i=0; i < len ; ++i) 
            for (int j=i; j < len; ++j) {
                int k = strncmp(sheetStrandId[i].first,
                                sheetStrandId[j].first,3);
                if ( (k != 0) ||
                     ((k == 0) &&
                      (sheetStrandId[i].second != sheetStrandId[j].second)))
                    return false;
            }
    }
    return same;
}

short SecondStruct::numOfAHelix(const int resNum) const
{
  for (Helices::iterator it = const_cast<Helix*>(alpha.begin());
       it != alpha.end(); ++it)
      if ((*it).isResidueInHelix(resNum))
          //if (isInAHelix(it-alpha.begin()+1, resNum))
          return (it-alpha.begin()+1);
  return -1;
}

const char * SecondStruct::numOfBSheet(const int resNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    if (isInBSheet(it->sheetId(), resNum))
      return (it->sheetId());
  return '\0';
}


pair<const char *,short> SecondStruct::sheetAndStrandId(const int resNum) const
{
  for (Sheets::iterator it = const_cast<Sheet*>(beta.begin()); 
       it != beta.end(); ++it) 
    for (Sheet::iterator jt = (*it).begin(); jt != (*it).end(); ++jt) 
        if ((*jt).isResInStrand(resNum)) {
            return pair<const char *,short>(it->sheetId(), jt->strandNumber());
        }
  return pair<const char *,short>((const char *)NULL,-1);
}
 
void SecondStruct::readSSFromPDBFile(ifstream& PDBFileStream)
{
  char line[85];
  int j=0;
  while (PDBFileStream.get(line,85,'\n')) {
    if (HelixPDB::isHelix(line)) {
      alpha.push_back(Helix(line));
    }
    else if (StrandPDB::isStrand(line)) {    
      if (StrandPDB::sense(line)==0 || StrandPDB::currAtom(line) == ' ') {
	// first strand
	int max_size = StrandPDB::numOfStrandsInSheet(line);
	
	Sheet bs(line);
	for (int i=1; i < max_size; ++i) {
	  PDBFileStream.get(*line);
	  PDBFileStream.get(line,85,'\n');
	  
	  //assert(StrandPDB::isStrand(line));
	  //bs.add(line);
	  
	  if (!StrandPDB::isStrand(line)) {
	    cerr << "at least one strand is missing = " << endl;
	    break;
	  } else {	    
	    bs.add(line);
	  }
	}
	
	beta.push_back(bs);
	++j;
      }
    }
    else if (PDB::isATOMrec(line)) {
      /* if we have reached
	 the atoms no change there 
	 will be any information on the 
	 helices afterwards */
        break;
    }

    PDBFileStream.get(*line);
  }
}


void SecondStruct::readSSFromDsspFile(const char* const dsspFileName,
				      vector<Helix>& helices,
				      vector<Strand>& strands) {
  ifstream dsspFileStream(dsspFileName);
  if (!dsspFileStream) {
    cerr << "couldn't open the following DSSP file " << dsspFileName << endl;
    exit(1);
  }
  
  readSSFromDsspFile(dsspFileStream, helices, strands);
  dsspFileStream.close();  
}

void SecondStruct::readSSFromDsspFile(ifstream& dsspFileStream,
				      vector<Helix>& helices,
				      vector<Strand>& strands) {
  char line[DSSP_MAX_LENGTH_OF_RECORD];
  
  // Skip the DSSP Header
  bool isHeaderFinished = false;
  while (!dsspFileStream.eof() && !isHeaderFinished) {
    dsspFileStream.get(line, DSSP_MAX_LENGTH_OF_RECORD, '\n');
    isHeaderFinished = (strncmp(line, "  #  RESIDUE", 10) == 0);
    
    // Clear new-line from the stream
    dsspFileStream.get(*line); 
  }
  
  if (dsspFileStream.eof()) {
    cerr << "Cannot construct SecondStruct because of an illegal DSSP file" 
	 << endl;
    exit(1);
  } 

  // The attributes of the current scanned secondary structure
  short helixNum = -1;
  short strandNum = -1;
  int startResidue = -1; 
  int endResidue = -1;
  short length = -1;
  char startChain = ' ';
  char endChain = ' ';
  char ssType = ' ';
  char previousSSType = ' ';
  
  while (!dsspFileStream.eof()) {
    dsspFileStream.get(line, DSSP_MAX_LENGTH_OF_RECORD, '\n');
    
    ssType = line[DSSP_SS_TYPE_FIELD];
    
    if (previousSSType != ssType) {
      if (previousSSType == DSSP_HELIX_SS_TYPE) {
	// The previous record was the last record of a helix
	Helix helix(helixNum, startResidue, endResidue,  
		    startChain, endChain , length,
		    HelixPDB::Unknown);
	helices.push_back(helix);
      } else if (previousSSType == DSSP_STRAND_SS_TYPE) {
	// The previous record was the last record of a strand
	Strand strand(strandNum, startResidue, endResidue, 
		      startChain, endChain, StrandPDB::FirstStrand, 
		      StrandConnection());

        strands.push_back(strand);
      } else {
	// start scanning a new secondary structure
	if (ssType == DSSP_HELIX_SS_TYPE) {
	  // start scanning a helix
	  helixNum++;
	  length = 1;
	  
	  startResidue = atoi(line + DSSP_START_OF_PDB_RESIDUE_FIELD);
	  endResidue = startResidue;
	  
	  startChain = line[DSSP_CHAIN_FIELD];
	  endChain = startChain;
	} else if (ssType == DSSP_STRAND_SS_TYPE) {
	// start scanning a beta strand
	strandNum++;
	length = 1;
	
	startResidue = atoi(line + DSSP_START_OF_PDB_RESIDUE_FIELD);
	endResidue = startResidue;
	
	startChain = line[DSSP_CHAIN_FIELD];
	endChain = startChain;
	}
      } 
    } else { //previousSSType == ssType
      endResidue = atoi(line + DSSP_START_OF_PDB_RESIDUE_FIELD);
      endChain = line[DSSP_CHAIN_FIELD];
      length++;
    }
  
    previousSSType = ssType;

    // Clear new-line from the stream
    dsspFileStream.get(*line); 
  }
}


void SecondStruct::readSSFromPDBFile(const char* const pdbFileName,
				     vector<Helix>& helices,
				     vector<Strand>& strands) { 
  ifstream pdbFileStream(pdbFileName);
    if (!pdbFileStream) {
        cerr << "couldn't open the following PDB file " << pdbFileName << endl;
        exit(1);
    }

    readSSFromPDBFile(pdbFileStream, helices, strands);
    pdbFileStream.close();  
}


void SecondStruct::readSSFromPDBFile(ifstream& pdbFileStream,
				     vector<Helix>& helices,
				     vector<Strand>& strands) {
  char line[PDB_MAX_LENGTH_OF_RECORD];

  while (pdbFileStream.get(line, PDB_MAX_LENGTH_OF_RECORD, '\n')) {
    if (HelixPDB::isHelix(line)) {
      helices.push_back(Helix(line));
    } else if (StrandPDB::isStrand(line)) {    
      if ( (StrandPDB::sense(line) == 0) || (StrandPDB::currAtom(line) == ' ') ) {
	// first strand
	int max_size = StrandPDB::numOfStrandsInSheet(line);
	
	for (int i = 1; i < max_size ; ++i) {
	  // Clear new-line from the stream
	  pdbFileStream.get(*line);
	  pdbFileStream.get(line, PDB_MAX_LENGTH_OF_RECORD,'\n');
	  
	  if (!StrandPDB::isStrand(line)) {
	    cerr << "at least one strand is missing = " << endl;
	    break;
	  } else {	
	    strands.push_back(Strand(line));
	  }
	}
      }
    } else if (PDB::isATOMrec(line)) {
      // if we have reached the atoms no change there will be any
      // information on the helices afterwards
      break;
    }
    
    // Clear new-line from the stream
    pdbFileStream.get(*line);
  }
}


ostream& operator<<(ostream& s, const SecondStruct& ss)
{
  for (Helices::iterator it = const_cast<Helix*>(ss.alpha.begin());
       it != ss.alpha.end(); ++it)
    s << *it << endl;
  for (Sheets::iterator it = const_cast<Sheet*>(ss.beta.begin());
       it != ss.beta.end(); ++it) {
      s << *it << endl;
  }
  return s;
}
  
