#include "StrandPDB.h"
#include "string.h"
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include "iostream.h"

typedef StrandPDB::RelativeParallelism RelativeParallelism;

// GROUP: relative parallelism between the strand in the sheets defintion
  const short StrandPDB::Parallel = 1;
  const short StrandPDB::AntiParallel = -1;
  const short StrandPDB::FirstStrand = 0;
    
  // GROUP: SHEET record constants and methods

  const short StrandPDB::strandNumberField = 7;
  const short StrandPDB::sheetIDField = 11;
  const short StrandPDB::numStrandsField = 14;
  const short StrandPDB::initResNameField = 17;
  const short StrandPDB::initChainIdField = 21;
  const short StrandPDB::initSeqNumField = 22;
  const short StrandPDB::initICodeField = 26;
  const short StrandPDB::endResNameField = 28;
  const short StrandPDB::endChainIdField = 32;
  const short StrandPDB::endSeqNumField = 33;
  const short StrandPDB::endICodeField = 37;
  const short StrandPDB::senseField = 38;
  const short StrandPDB::curAtomField = 41;
  const short StrandPDB::curResNameField =45;
  const short StrandPDB::curChainIdField = 49;
  const short StrandPDB::curResSeqField = 50;
  const short StrandPDB::curICodeField = 54;
  const short StrandPDB::prevAtomField = 56;
  const short StrandPDB::prevResNameField = 60;
  const short StrandPDB::prevChainIdField = 64;
  const short StrandPDB::prevResSeqField = 65;
  const short StrandPDB::prevICodeField = 69;

bool StrandPDB::isStrand(const char* const PDBrec)
{
  return (strncmp(PDBrec,"SHEET",5)==0);
}

short StrandPDB::strandNum(const char* const PDBrec)
{
  return atoi(PDBrec+strandNumberField);
}
void StrandPDB::strandSheetId(const char* const PDBrec, char id[])
{
    
   strncpy(id, PDBrec+sheetIDField,3);
}

short StrandPDB::numOfStrandsInSheet(const char* const PDBrec)
{
  return atoi(PDBrec+numStrandsField);
}
void StrandPDB::initResName(const char* const PDBrec, char res[])
{
  strncpy(res,PDBrec+initResNameField,3);
  res[3]='\0';
}
char StrandPDB::initChainId(const char* const PDBrec)
{
  return PDBrec[initChainIdField];
}
int StrandPDB::initSeqNum(const char* const PDBrec)
{
  return atoi(PDBrec+initSeqNumField);
}
char StrandPDB::initICode(const char* const PDBrec)
{
  return PDBrec[initICodeField];
}

void StrandPDB::endResName(const char* const PDBrec,char res[])
{
  strncpy(res, PDBrec+endResNameField,3);
  res[4]='\0';
}

char StrandPDB::endChainId(const char* const PDBrec)
{
  return PDBrec[endChainIdField];
}
int StrandPDB::endSeqNum(const char* const PDBrec)
{
  return atoi(PDBrec+endSeqNumField);
}
char StrandPDB::endICode(const char* const PDBrec)
{
  return PDBrec[endICodeField];
}
RelativeParallelism StrandPDB::sense(const char* const PDBrec)
{
  if (PDBrec[senseField] == '-' && PDBrec[senseField+1]=='1')
    return AntiParallel;
  else if (PDBrec[senseField+1] == '1')
    return Parallel;
  else  /* might need to add here info 
           regarding the REMARK records */
    return FirstStrand;
}
char StrandPDB::currAtom(const char* const PDBrec)
{
  // might be in a different level like the strand 
  // and sheet level
  
  // removing the assert and changing it to a warning
  //assert(PDBrec[curAtomField]=='N' || PDBrec[curAtomField]=='O');
  if (!(PDBrec[curAtomField]=='N' || PDBrec[curAtomField]=='O')) 
    cerr << "might be wrong data in line: " << PDBrec << endl
         << "atoms of connection must be either N or O\n";
  
  return PDBrec[curAtomField];
}
void StrandPDB::currResName(const char* const PDBrec, char res[])
{
  strncpy(res, PDBrec+curResNameField, 3);
  res[3]='\0';
}

char StrandPDB::currChainId(const char* const PDBrec)
{
  return PDBrec[curChainIdField];
}
int StrandPDB::currResSeq(const char* const PDBrec)
{
  return atoi(PDBrec+curResSeqField);
}
char StrandPDB::currICode(const char* const PDBrec)
{
  return PDBrec[curICodeField];
}
char StrandPDB::prevAtom(const char* const PDBrec)
{
  // same notice as above
  // assert(PDBrec[prevAtomField]=='N' || PDBrec[prevAtomField]=='O');
  if (!(PDBrec[curAtomField]=='N' || PDBrec[curAtomField]=='O')) 
    cerr << "might be wrong data in line: " << PDBrec << endl
         << "atoms of connection must be either N or O\n";
  return PDBrec[prevAtomField];
}

void StrandPDB::prevResName(const char* const PDBrec, char res[])
{
  strncpy(res, PDBrec+prevResNameField,3);
  res[3]='\0';
}

char StrandPDB::prevChainId(const char* const PDBrec)
{
  return PDBrec[prevChainIdField];
}

int StrandPDB::prevResSeq(const char* const PDBrec)
{
  return atoi(PDBrec+prevResSeqField);
}

char StrandPDB::prevICode(const char* const PDBrec)
{
  return PDBrec[prevICodeField];
}











