#include "Shou.h"
#include "Surface.h"

Surface::Surface() : Molecule<SurfacePoint>()
{}
   
int Surface::readShouFile(istream& shouStream)
{
  char line[85];
  while (!shouStream.eof())  {
    shouStream.get(line, 85, '\n');
    if (Shou::isRec(line))
      add(SurfacePoint(line));
    shouStream.get(*line);   // Clear new-line from stream.
  }
  return size();
}

void Surface::outputVRML(ostream& VRMLstream,
                         const float red /*= 1.0*/,
                         const float green /*= 1.0*/,
                         const float blue /*= 1.0*/) const
{
  const char header [] = "#VRML V1.0 ascii";  // VRML 1.0 file header
  const char separator[] = "Separator {\n";   // VRML spearator
  const char ind[] = "  ";                    // indentation
  
  VRMLstream << header << '\n' << '\n'; 
  VRMLstream << separator;
  VRMLstream << ind << "Material {\n";
  VRMLstream << ind << ind << "diffuseColor " 
             << red << ' ' << green << ' ' << blue << '\n';
  VRMLstream << ind << "}\n";
  
  VRMLstream << ind << "Coordinate3 {\n";
  VRMLstream << ind << ind << "point [\n";
  for (const_iterator it=begin(); it != end(); ++it) {
    if (it != begin())
      VRMLstream << ',' << '\n';
    const Vector3 pos = (*it).position();
    const Vector3 tip = pos + (*it).normal();
    VRMLstream << ind << ind << ind  
               << pos[0] << ' ' << pos[1] << ' ' << pos[2] << ',' << '\n';
    VRMLstream << ind << ind << ind 
               << tip[0] << ' ' << tip[1] << ' ' << tip[2];
  }
  VRMLstream << '\n';
  VRMLstream << ind << ind << ']' << '\n';
  VRMLstream << ind << '}' << '\n';
  VRMLstream << ind << "IndexedLineSet {\n";
  VRMLstream << ind << ind << "coordIndex [\n";

  for (unsigned i=0; i<size(); ++i) {
    VRMLstream << ind << ind << ind << i*2 << ", " << i*2+1 << ", " << -1; 
    if (i != size())
      VRMLstream << ',';
    VRMLstream << '\n';
  }

  VRMLstream << ind << ind << ']' << '\n';
  VRMLstream << ind << '}' << '\n';
  VRMLstream << '}' << '\n';
}

ostream& operator<<(ostream& s, const Surface& surf)
{

#ifdef __Debug
  s << "address=" << &surf <<endl;
  s << "size =" << surf.size() << endl;
#endif

  for (Surface::const_iterator it=surf.begin(); it != surf.end() ; ++it)
    s << *it ;
  s << endl;
  return s;
}
