#include "Timer.h"


Timer::Timer()
  : id(NULL)
{
  reset();
}

Timer::Timer(const char* const tmID)
{
  unsigned n=strlen(tmID);
  id = new char[n+1];
  strcpy(id,tmID);
  reset();
}

Timer::~Timer()
{
  if (id)
    delete [] id;
}

void Timer::reset()
{
  begin_clock = clock();
  clock_sum = 0;
}

void Timer::start()
{
  begin_clock = clock();
}

void Timer::stop()
{
  if (begin_clock >= 0) {
    clock_sum+= clock() - begin_clock;
    begin_clock = -1;
  }
}

float Timer::operator()() const
{
  if (begin_clock >= 0)
    return (clock_sum + (clock() - begin_clock))/(float)CLOCKS_PER_SEC;
  else
    return clock_sum/(float)CLOCKS_PER_SEC;
}

ostream& operator<<(ostream& out, const Timer& tm)
{
  if (tm.id)
    out << tm.id << ": ";
  out.setf(ios::fixed, ios::floatfield);
  out.setf(ios::right, ios::adjustfield);
  out.precision(2);
  out.width(8);
  unsigned int time = (unsigned int)(tm()*100.0);
  unsigned int sec100 = time%100;
  time /= 100;
  unsigned int sec = time%60;
  time /= 60;
  unsigned int min = time%60;
  time /= 60;
  out.fill('0');
  out.width(2);   out << time << ':';
  out.width(2);   out << min << ':'; 
  out.width(2);   out << sec << '.'; 
  out.width(2);   out << sec100;
  out.fill(' ');
  return out;
}






