#include <stdlib.h>
#include "math.h"

#include "Vector3.h"

static const char rcsid[] = "$Id: Vector3.cc,v 1.1 1997/06/02 16:23:15 meirfux Exp $";

typedef Vector3::real real;

Vector3::Vector3()
{
  c[0] = c[1] = c[2] = 0;
}

Vector3::Vector3(const real &nx, const real &ny, const real &nz)
{
  c[0]=nx;
  c[1]=ny;
  c[2]=nz;
}

Vector3::Vector3(const real x[3])
{
  c[0] = x[0];
  c[1] = x[1];
  c[2] = x[2];
}

Vector3::Vector3(const double x[3])
{
  c[0] = x[0];
  c[1] = x[1];
  c[2] = x[2];
}

  
real Vector3::x() const
{
  return c[0];
}

real Vector3::y() const
{
  return c[1];
}

real Vector3::z() const
{
  return c[2];
}

real Vector3::x(const unsigned short coord) const
{
  return c[coord-1];
}

void Vector3::updateX(const real& x) {
  c[0]=x;
}
void Vector3::updateY(const real& y) {
  c[1]=y;
}
void Vector3::updateZ(const real& z) {
  c[2]=z;
}

real Vector3::norm2() const
{
  return c[0]*c[0]+c[1]*c[1]+c[2]*c[2];
}

real Vector3::norm() const
{
  return sqrt(norm2());
}

real Vector3::dist2(const Vector3 &p) const
{
  real d=c[0]-p.c[0];  
  real s=d*d;
  d=c[1]-p.c[1];
  s+=d*d;
  d=c[2]-p.c[2];
  return s+d*d;
}

real Vector3::dist(const Vector3 &p) const
{
  return sqrt(dist2(p));
}

//const real& Vector3::operator[](const unsigned short coord) const
//{
//  return c[coord];
//}

Vector3 operator+(const Vector3 &p1, const Vector3 &p2)
{
  return Vector3(p1.c[0]+p2.c[0], p1.c[1]+p2.c[1], p1.c[2]+p2.c[2]);
}

Vector3 operator-(const Vector3 &p1, const Vector3 &p2)
{
  return Vector3(p1.c[0]-p2.c[0], p1.c[1]-p2.c[1], p1.c[2]-p2.c[2]);
}
 
Vector3 operator-(const Vector3 &p)
{
  return Vector3(-p.c[0], -p.c[1], -p.c[2]);
}
 
real operator*(const Vector3 &p1, const Vector3 &p2)
{
  return p1.c[0]*p2.c[0]+p1.c[1]*p2.c[1]+p1.c[2]*p2.c[2];
}

Vector3 operator*(const Vector3 &p, const real &m)
{
  return Vector3(p.c[0]*m, p.c[1]*m, p.c[2]*m);
}

Vector3 operator*(const real &m, const Vector3 &p)
{
  return Vector3(p.c[0]*m, p.c[1]*m, p.c[2]*m);
}

Vector3 operator/(const Vector3 &p, const real &m)
{
  return Vector3(p.c[0]/m, p.c[1]/m, p.c[2]/m);
}

Vector3 operator&(const Vector3 &p1, const Vector3 &p2)
{
  return Vector3(p1.c[1]*p2.c[2]-p1.c[2]*p2.c[1],
		 p1.c[2]*p2.c[0]-p1.c[0]*p2.c[2],
		 p1.c[0]*p2.c[1]-p1.c[1]*p2.c[0]);
}

real operator|(const Vector3 &p1, const Vector3 &p2)
{
  return p1.dist(p2);
}

real operator^(const Vector3 &p1, const Vector3 &p2)
{
  return acos(p1*p2/(p1.norm()*p2.norm()));
}

Vector3& Vector3::operator+=(const Vector3 &p)
{
  c[0]+=p.c[0];  c[1]+=p.c[1];  c[2]+=p.c[2];
  return *this;
}

Vector3& Vector3::operator-=(const Vector3 &p)
{
  c[0]-=p.c[0];  c[1]-=p.c[1];  c[2]-=p.c[2];
  return *this;
}

Vector3& Vector3::operator*=(const real &m)
{
  c[0]*=m;  c[1]*=m;  c[2]*=m;
  return *this;
}

Vector3& Vector3::operator/=(const real &m)
{
  c[0]/=m;  c[1]/=m;  c[2]/=m;
  return *this;
}

ostream& operator<<(ostream& s, const Vector3 &v)
{
  return s << v.c[0] << ' ' << v.c[1] << ' ' << v.c[2];
}

istream& operator>>(istream& s, Vector3 &v)
{
  return s >> v.c[0] >> v.c[1] >> v.c[2];
}







